// Prevent "more" messages from appearing
   set more off
// Control line length
   set linesize 150

///////////////////////////////////////////////////////////////////////////////////////
////////               BEGIN DATA IMPORT AND PREPARATION FOR EXAMPLE 4          ///////
////////                  CHANGE "filesave" TO YOUR FOLDER PATH                 ///////
///////////////////////////////////////////////////////////////////////////////////////

// Define global variable for file location to be replaced in code below
// \\Client\ precedes path in Virtual Desktop outside H drive
   global filesave "C:\Dropbox\23_PSQF6272\PSQF6272_Example8b"

// Open trimmed example excel data file from sheet "grade10" and clear away existing data
   clear // clear memory in case of open data
   import excel "$filesave\Example8b_Data.xlsx", firstrow case(preserve) sheet("grade10") clear 
   
// Open external file to save results to
   log using $filesave\Example8b_STATA_Output.log, replace   

// Add labels to original variables
   label variable districtID "districtID: District ID number"
   label variable studentID  "studentID: Student ID number"
   label variable schoolID   "schoolID: School ID number"
   label variable lunch      "lunch: 0=Paid, 1=Reduced, 2=Free"
   label variable math       "math: Math Test Score"

display "STATA Descriptive Statistics within Student-Level Data"
tabulate lunch
       
// Create new dummy-coded predictors (start as blank, then recode)
   gen PvRF=.  
   gen PRvF=.
   replace PvRF=0 if lunch==0  // Replace each for paid lunch
   replace PRvF=0 if lunch==0
   replace PvRF=1 if lunch==1  // Replace each for reduced lunch
   replace PRvF=0 if lunch==1
   replace PvRF=1 if lunch==2  // Replace each for free lunch
   replace PRvF=1 if lunch==2
   label variable PvRF "PvRF: 0=Paid, 1=Reduced or Free Lunch"
   label variable PRvF "PRvF: 0=Paid or Reduced, 1=Free Lunch"
     
// Filter to complete cases before computing district and school means
   egen nmiss=rowmiss(math lunch)
   drop if nmiss>0
       
// Compute level-2 school means for level-1 variables
   egen schoolN = count(math), by(schoolID)
   egen SM_math = mean(math),  by(schoolID)     
   egen SM_PvRF = mean(PvRF),  by(schoolID)     
   egen SM_PRvF = mean(PRvF),  by(schoolID) 
     
display "STATA Descriptive Statistics within School-Level-2 Data"
preserve  // Save for later use, then compute school-level dataset
collapse  schoolN SM_math SM_PvRF SM_PRvF, by(schoolID)
summarize schoolN SM_math SM_PvRF SM_PRvF
restore   // Go back to student-level dataset  
	
// Compute level-3 district means for level-1 variables
   egen districtN = count(math), by(districtID)
   egen DM_math   = mean(math),  by(districtID)     
   egen DM_PvRF   = mean(PvRF),  by(districtID)     
   egen DM_PRvF   = mean(PRvF),  by(districtID) 
     
display "STATA Descriptive Statistics within District-Level-3 Data"
preserve  // Save for later use, then compute school-level dataset
collapse  districtN DM_math DM_PvRF DM_PRvF, by(districtID)
summarize districtN DM_math DM_PvRF DM_PRvF	
tabulate districtN 
restore   // Go back to student-level dataset  
	
// Cluster-mean-center level-1 variables at level-2 means
   gen WS_PvRF = PvRF - SM_PvRF
   gen WS_PRvF = PRvF - SM_PRvF
   gen WS_math = math - SM_math
   label variable WS_PvRF "WS_PvRF: Within-School Paid vs Reduced/Free Lunch"
   label variable WS_PRvF "WS_PRvF: Within-School Paid/Reduced vs Free Lunch"
   label variable WS_math "WS_math: Within-School Math"
// Cluster-mean-center level-2 variables at level-3 means
   gen WD_PvRF = SM_PvRF - DM_PvRF
   gen WD_PRvF = SM_PRvF - DM_PRvF
   gen WD_math = SM_math - DM_math
   label variable WD_PvRF "WD_PvRF: Within-District Paid vs Reduced/Free Lunch"
   label variable WD_PRvF "WD_PRvF: Within-District Paid/Reduced vs Free Lunch"
   label variable WD_math "WD_math: Within-District Math"
// Center district means so ref school has 0=.72, 1=.09, and 2=.19
   gen DM_PvRF28 = DM_PvRF - .28
   gen DM_PRvF19 = DM_PRvF - .19

display "STATA Descriptive Statistics within Student-Level-1 Data"
summarize WS_PvRF WS_PRvF WD_PvRF WD_PRvF DM_PvRF DM_PRvF
pwcorr math SM_math DM_math WS_math WD_math, sig


///////////////////////////////////////////////////////////////////////////////////////
////////                     BEGIN MODELS FOR EXAMPLE 8b                         //////
///////////////////////////////////////////////////////////////////////////////////////

// Empty models

display "STATA Model 1a: Empty Means, Two-Level Random Intercept for Math"
mixed math , || schoolID: , reml dfmethod(satterthwaite) dftable(pvalue) nolog
display "-2LL = " e(ll)*-2      // Print -2LL for model  
estat icc                       // Intraclass correlation
estimates store Fit1a           // Save for LRT

display "STATA Model 1b: Empty Means, Three-Level Random Intercept for Math"
mixed math , || districtID:  || schoolID: , ///
      reml dfmethod(satterthwaite) dftable(pvalue) nolog difficult
display "-2LL = " e(ll)*-2      // Print -2LL for model  
estat icc                       // Intraclass correlation
estimates store Fit1b           // Save for LRT
lrtest Fit1b Fit1a              // Need level-3 random intercept?

display "STATA: Empty Means, Three-Level Random Intercept for Ordinal Lunch"
display "Provides submodel thresholds (multiply by -1 to get intercepts)"
meologit lunch , || districtID:  || schoolID: , nolog
display "-2LL = " e(ll)*-2      // Print -2LL for model  
display "ICC L2 = " (1.194827+0.326018)/(1.194827+0.326018+3.29)  
display "ICC L3 = " 1.194827/(1.194827+0.326018)
          
// Add Paid vs Reduced/Free Lunch predictor slopes           
                
display "STATA Model 2a: Add Variable-Centered Lunch Predictors at all 3 Levels"
mixed math c.WS_PvRF c.WS_PRvF c.WD_PvRF c.WD_PRvF c.DM_PvRF28 c.DM_PRvF19,  ///
      || districtID:  || schoolID: , reml dfmethod(satterthwaite) dftable(pvalue) nolog difficult
display "-2LL = " e(ll)*-2        // Print -2LL for model  
estimates store Fit2a             // Save fit for LRT
lincom c.WS_PvRF*-1 + c.WD_PvRF*1    // L2 PvRF Context
lincom c.WS_PRvF*-1 + c.WD_PRvF*1    // L2 PRvF Context
lincom c.WD_PvRF*-1 + c.DM_PvRF28*1  // L3 PvRF Context
lincom c.WD_PRvF*-1 + c.DM_PRvF19*1  // L3 PRvF Context

display "STATA Model 2b: Add L2 Random Slope for Cluster-MC WS_PvRF"
mixed math c.WS_PvRF c.WS_PRvF c.WD_PvRF c.WD_PRvF c.DM_PvRF28 c.DM_PRvF19,  ///
      || districtID:  || schoolID: WS_PvRF, cov(un) reml dfmethod(satterthwaite) dftable(pvalue) nolog difficult
display "-2LL = " e(ll)*-2     // Print -2LL for model  
estimates store Fit2b          // Save fit for LRT
lrtest Fit2b Fit2a             // Need L2 random slope?
estat recovariance, relevel(schoolID) correlation  // L2 random effect correlation

display "STATA Model 2c: Add L3 Random Slope for Cluster-MC WS_PvRF"
mixed math c.WS_PvRF c.WS_PRvF c.WD_PvRF c.WD_PRvF c.DM_PvRF28 c.DM_PRvF19,  ///
      || districtID: WS_PvRF, cov(un)  || schoolID: WS_PvRF, cov(un)         /// 
		reml dfmethod(satterthwaite) dftable(pvalue) nolog difficult
display "-2LL = " e(ll)*-2       // Print -2LL for model  
estat recovariance, relevel(districtID) correlation  // L3 random effect correlation
estat recovariance, relevel(schoolID)   correlation  // L3 random effect correlation
estimates store Fit2c            // Save fit for LRT
//lrtest Fit2c Fit2b            // Need L3 random slope?


// Close external file of saved results
   log close

